#include "stm32f4xx.h"
#include "stdint.h"
#include "RPM_Encoder.h"
#include "delay.h"
const int AF02=0x02;
uint32_t encoder_counts;
float rpm;
uint32_t timer_counter;
volatile uint8_t rpm_new_data,direction;

void Encoder_init(uint16_t max_count)
	{
	RCC->APB1ENR|=RCC_APB1ENR_TIM3EN;
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOBEN;
	encoder_counts=max_count;
	/*GPIO B configuration for timer mode */
	GPIOB->MODER|=GPIO_MODER_MODE4_1|GPIO_MODER_MODE5_1;
	GPIOB->MODER&=~(GPIO_MODER_MODE4_0|GPIO_MODER_MODE5_0);
	GPIOB->AFR[0]|=(AF02<<16)|(AF02<<20);

	/*Timer configuration for encoder mode */
	TIM3->ARR = 0xFFFF; //set the max count
	/*Timer in encoder mode*/
	TIM3->CCMR1 |= (TIM_CCMR1_CC1S_0 | TIM_CCMR1_CC2S_0 );
	TIM3->CCER &= ~(TIM_CCER_CC1P | TIM_CCER_CC2P);
	TIM3->SMCR |= TIM_SMCR_SMS_0 | TIM_SMCR_SMS_1;

	/*Timer 5 to generate interrupt each 100ms*/

	RCC->APB1ENR|=RCC_APB1ENR_TIM5EN;
	TIM5->PSC=16000-1;
	TIM5->ARR=1000-1;
	TIM5->DIER|=TIM_DIER_UIE;
	NVIC_EnableIRQ(TIM5_IRQn);
	TIM5->CR1|=TIM_CR1_CEN;

	/*Start the timer */
	TIM3->CR1|=TIM_CR1_CEN;

	}

float get_rpm()
{
	rpm_new_data=no_new_rpm;
	rpm=(float)(((float)timer_counter)/(float)encoder_counts)*(float)60.0;
	return rpm;
}



uint8_t new_rpm_available()
{
	uint8_t return_data;
	if(rpm_new_data==new_rpm){return_data=new_rpm;}
	else {return_data=no_new_rpm;}
	return return_data;

}

uint8_t get_direction()
{

	return direction;
}




void TIM5_IRQHandler(void)
{
	TIM5->SR=0; /*clear the status register*/
	/*Determine the direction
	 * 0-> Clockwise
	 * 1->Counter Clockwise
	 * */
	if((TIM3->CR1 & TIM_CR1_DIR)>>TIM_CR1_DIR_Pos==0)
	{
		direction=ClockWise;
		timer_counter=TIM3->CNT;
		TIM3->CNT=0;
	}
	else
	{
		direction=CounterClockWise;
		timer_counter=0xFFFF-TIM3->CNT;
		TIM3->CNT=0xFFFF;
	}

	rpm_new_data=new_rpm;
}

